@@ -74,7 +74,10 @@ pub enum Error {
     #[error("The etag for this request is missing or invalid")]
     InvalidEtag,
 
-    /* Integration issues */
+    /* Token issues */
+    /// Token invalid or expired
+    #[error("Invalid CSRF token")]
+    BadToken,
     /// Unable to fetch a CSRF token
     #[error("Unable to get token `{0}`")]
     TokenError(String),
@@ -111,6 +114,8 @@ pub enum Error {
     EditConflict(String),
 
     /* Catchall/generic issues */
+    #[error("{info}, retry-after: {retry_after}")]
+    MaxlagError { info: String, retry_after: u32 },
     /// Any arbitrary error returned by the MediaWiki API
     #[error("{0}")]
     ApiError(ApiError),
@@ -132,7 +137,14 @@ impl From<ApiError> for Error {
         match apierr.code.as_str() {
             "assertuserfailed" => Self::NotLoggedIn,
             "assertbotfailed" => Self::NotLoggedInAsBot,
+            "badtoken" => Self::BadToken,
             "login-throttled" => Self::LoginThrottled,
+            "maxlag" => Self::MaxlagError {
+                info: apierr.text,
+                // Since Retry-After header isn't accessible here, just set it
+                // to 0
+                retry_after: 0,
+            },
             "wrongpassword" => Self::WrongPassword,
             _ => Self::ApiError(apierr),
         }
